@@ -48,13 +48,9 @@
 #include "amdgpu_bo_helper.h"
 #include "amdgpu_version.h"
 
-#include "amdgpu_list.h"
-
+#include <list.h>
 #include <xf86Priv.h>
-
-#if DRI2INFOREC_VERSION >= 9
-#define USE_DRI2_PRIME
-#endif
+#include <X11/extensions/dpmsconst.h>
 
 #define FALLBACK_SWAP_DELAY 16
 
@@ -124,6 +120,7 @@ amdgpu_dri2_create_buffer2(ScreenPtr pScreen,
 			cpp = 2;
 			break;
 		case 24:
+		case 30:
 			cpp = 4;
 			break;
 		default:
@@ -164,29 +161,28 @@ amdgpu_dri2_create_buffer2(ScreenPtr pScreen,
 						   AMDGPU_CREATE_PIXMAP_DRI2);
 	}
 
+	if (!pixmap)
+		return NULL;
+
 	buffers = calloc(1, sizeof *buffers);
-	if (buffers == NULL)
+	if (!buffers)
 		goto error;
 
-	if (pixmap) {
-		if (is_glamor_pixmap) {
-			pixmap = amdgpu_glamor_set_pixmap_bo(drawable, pixmap);
-			pixmap->refcnt++;
-		}
-
-		if (!amdgpu_get_flink_name(pAMDGPUEnt, pixmap, &buffers->name))
-			goto error;
+	if (is_glamor_pixmap) {
+		pixmap = amdgpu_glamor_set_pixmap_bo(drawable, pixmap);
+		pixmap->refcnt++;
 	}
 
+	if (!amdgpu_get_flink_name(pAMDGPUEnt, pixmap, &buffers->name))
+		goto error;
+
 	privates = calloc(1, sizeof(struct dri2_buffer_priv));
-	if (privates == NULL)
+	if (!privates)
 		goto error;
 
 	buffers->attachment = attachment;
-	if (pixmap) {
-		buffers->pitch = pixmap->devKind;
-		buffers->cpp = cpp;
-	}
+	buffers->pitch = pixmap->devKind;
+	buffers->cpp = cpp;
 	buffers->driverPrivate = privates;
 	buffers->format = format;
 	buffers->flags = 0;	/* not tiled */
@@ -198,19 +194,10 @@ amdgpu_dri2_create_buffer2(ScreenPtr pScreen,
 
 error:
 	free(buffers);
-	if (pixmap)
-		(*pScreen->DestroyPixmap) (pixmap);
+	(*pScreen->DestroyPixmap) (pixmap);
 	return NULL;
 }
 
-DRI2BufferPtr
-amdgpu_dri2_create_buffer(DrawablePtr pDraw, unsigned int attachment,
-			  unsigned int format)
-{
-	return amdgpu_dri2_create_buffer2(pDraw->pScreen, pDraw,
-					  attachment, format);
-}
-
 static void
 amdgpu_dri2_destroy_buffer2(ScreenPtr pScreen,
 			    DrawablePtr drawable, BufferPtr buffers)
@@ -239,11 +226,6 @@ amdgpu_dri2_destroy_buffer2(ScreenPtr pScreen,
 	}
 }
 
-void amdgpu_dri2_destroy_buffer(DrawablePtr pDraw, DRI2BufferPtr buf)
-{
-	amdgpu_dri2_destroy_buffer2(pDraw->pScreen, pDraw, buf);
-}
-
 static inline PixmapPtr GetDrawablePixmap(DrawablePtr drawable)
 {
 	if (drawable->type == DRAWABLE_PIXMAP)
@@ -273,17 +255,14 @@ amdgpu_dri2_copy_region2(ScreenPtr pScreen,
 	dst_drawable = &dst_private->pixmap->drawable;
 
 	if (src_private->attachment == DRI2BufferFrontLeft) {
-#ifdef USE_DRI2_PRIME
 		if (drawable->pScreen != pScreen) {
 			src_drawable = DRI2UpdatePrime(drawable, src_buffer);
 			if (!src_drawable)
 				return;
 		} else
-#endif
 			src_drawable = drawable;
 	}
 	if (dst_private->attachment == DRI2BufferFrontLeft) {
-#ifdef USE_DRI2_PRIME
 		if (drawable->pScreen != pScreen) {
 			dst_drawable = DRI2UpdatePrime(drawable, dest_buffer);
 			if (!dst_drawable)
@@ -291,7 +270,6 @@ amdgpu_dri2_copy_region2(ScreenPtr pScreen,
 			if (dst_drawable != drawable)
 				translate = TRUE;
 		} else
-#endif
 			dst_drawable = drawable;
 	}
 
@@ -319,14 +297,6 @@ amdgpu_dri2_copy_region2(ScreenPtr pScreen,
 	FreeScratchGC(gc);
 }
 
-void
-amdgpu_dri2_copy_region(DrawablePtr pDraw, RegionPtr pRegion,
-			DRI2BufferPtr pDstBuffer, DRI2BufferPtr pSrcBuffer)
-{
-	return amdgpu_dri2_copy_region2(pDraw->pScreen, pDraw, pRegion,
-					pDstBuffer, pSrcBuffer);
-}
-
 enum DRI2FrameEventType {
 	DRI2_SWAP,
 	DRI2_FLIP,
@@ -361,8 +331,9 @@ static void amdgpu_dri2_unref_buffer(BufferPtr buffer)
 {
 	if (buffer) {
 		struct dri2_buffer_priv *private = buffer->driverPrivate;
-		amdgpu_dri2_destroy_buffer(&(private->pixmap->drawable),
-					   buffer);
+		DrawablePtr draw = &private->pixmap->drawable;
+
+		amdgpu_dri2_destroy_buffer2(draw->pScreen, draw, buffer);
 	}
 }
 
@@ -465,9 +436,9 @@ xf86CrtcPtr amdgpu_dri2_drawable_crtc(DrawablePtr pDraw, Bool consider_disabled)
 static void
 amdgpu_dri2_flip_event_abort(xf86CrtcPtr crtc, void *event_data)
 {
-	AMDGPUInfoPtr info = AMDGPUPTR(crtc->scrn);
+	if (crtc)
+		AMDGPUPTR(crtc->scrn)->drmmode.dri2_flipping = FALSE;
 
-	info->drmmode.dri2_flipping = FALSE;
 	free(event_data);
 }
 
@@ -540,7 +511,6 @@ amdgpu_dri2_schedule_flip(xf86CrtcPtr crtc, ClientPtr client,
 	AMDGPUInfoPtr info = AMDGPUPTR(scrn);
 	struct dri2_buffer_priv *back_priv;
 	DRI2FrameEventPtr flip_info;
-	int ref_crtc_hw_id = drmmode_get_crtc_id(crtc);
 
 	flip_info = calloc(1, sizeof(DRI2FrameEventRec));
 	if (!flip_info)
@@ -560,10 +530,10 @@ amdgpu_dri2_schedule_flip(xf86CrtcPtr crtc, ClientPtr client,
 	/* Page flip the full screen buffer */
 	back_priv = back->driverPrivate;
 	if (amdgpu_do_pageflip(scrn, client, back_priv->pixmap,
-			       AMDGPU_DRM_QUEUE_ID_DEFAULT, flip_info,
-			       ref_crtc_hw_id,
+			       AMDGPU_DRM_QUEUE_ID_DEFAULT, flip_info, crtc,
 			       amdgpu_dri2_flip_event_handler,
-			       amdgpu_dri2_flip_event_abort)) {
+			       amdgpu_dri2_flip_event_abort, FLIP_VSYNC,
+			       target_msc - amdgpu_get_msc_delta(draw, crtc))) {
 		info->drmmode.dri2_flipping = TRUE;
 		return TRUE;
 	}
@@ -598,14 +568,6 @@ can_exchange(ScrnInfoPtr pScrn, DrawablePtr draw,
 	struct dri2_buffer_priv *back_priv = back->driverPrivate;
 	PixmapPtr front_pixmap;
 	PixmapPtr back_pixmap = back_priv->pixmap;
-	xf86CrtcConfigPtr xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
-	int i;
-
-	for (i = 0; i < xf86_config->num_crtc; i++) {
-		xf86CrtcPtr crtc = xf86_config->crtc[i];
-		if (crtc->enabled && crtc->rotatedData)
-			return FALSE;
-	}
 
 	if (!update_front(draw, front))
 		return FALSE;
@@ -629,17 +591,29 @@ can_exchange(ScrnInfoPtr pScrn, DrawablePtr draw,
 }
 
 static Bool
-can_flip(ScrnInfoPtr pScrn, DrawablePtr draw,
+can_flip(xf86CrtcPtr crtc, DrawablePtr draw,
 	 DRI2BufferPtr front, DRI2BufferPtr back)
 {
+	ScrnInfoPtr pScrn = crtc->scrn;
 	AMDGPUInfoPtr info = AMDGPUPTR(pScrn);
+	xf86CrtcConfigPtr config = XF86_CRTC_CONFIG_PTR(pScrn);
+	int num_crtcs_on;
+	int i;
 
-	return draw->type == DRAWABLE_WINDOW &&
-	    info->allowPageFlip &&
-	    !info->hwcursor_disabled &&
-	    !info->drmmode.present_flipping &&
-	    pScrn->vtSema &&
-	    DRI2CanFlip(draw) && can_exchange(pScrn, draw, front, back);
+	if (draw->type != DRAWABLE_WINDOW ||
+	    !info->allowPageFlip ||
+	    info->sprites_visible > 0 ||
+	    info->drmmode.present_flipping ||
+	    !pScrn->vtSema ||
+	    !DRI2CanFlip(draw))
+		return FALSE;
+
+	for (i = 0, num_crtcs_on = 0; i < config->num_crtc; i++) {
+		if (drmmode_crtc_can_flip(config->crtc[i]))
+			num_crtcs_on++;
+	}
+
+	return num_crtcs_on > 0 && can_exchange(pScrn, draw, front, back);
 }
 
 static void
@@ -721,7 +695,7 @@ static void amdgpu_dri2_frame_event_handler(xf86CrtcPtr crtc, uint32_t seq,
 
 	switch (event->type) {
 	case DRI2_FLIP:
-		if (can_flip(scrn, drawable, event->front, event->back) &&
+		if (can_flip(crtc, drawable, event->front, event->back) &&
 		    amdgpu_dri2_schedule_flip(crtc,
 					      event->client,
 					      drawable,
@@ -747,8 +721,8 @@ static void amdgpu_dri2_frame_event_handler(xf86CrtcPtr crtc, uint32_t seq,
 			box.x2 = drawable->width;
 			box.y2 = drawable->height;
 			REGION_INIT(pScreen, &region, &box, 0);
-			amdgpu_dri2_copy_region(drawable, &region, event->front,
-						event->back);
+			amdgpu_dri2_copy_region2(drawable->pScreen, drawable, &region,
+						 event->front, event->back);
 			swap_type = DRI2_BLIT_COMPLETE;
 		}
 
@@ -772,26 +746,6 @@ cleanup:
 	amdgpu_dri2_frame_event_abort(crtc, event_data);
 }
 
-drmVBlankSeqType amdgpu_populate_vbl_request_type(xf86CrtcPtr crtc)
-{
-	drmVBlankSeqType type = 0;
-	int crtc_id = drmmode_get_crtc_id(crtc);
-
-	if (crtc_id == 1)
-		type |= DRM_VBLANK_SECONDARY;
-	else if (crtc_id > 1)
-#ifdef DRM_VBLANK_HIGH_CRTC_SHIFT
-		type |= (crtc_id << DRM_VBLANK_HIGH_CRTC_SHIFT) &
-		    DRM_VBLANK_HIGH_CRTC_MASK;
-#else
-		ErrorF("amdgpu driver bug: %s called for CRTC %d > 1, but "
-		       "DRM_VBLANK_HIGH_CRTC_MASK not defined at build time\n",
-		       __func__, crtc_id);
-#endif
-
-	return type;
-}
-
 /*
  * This function should be called on a disabled CRTC only (i.e., CRTC
  * in DPMS-off state). It will calculate the delay necessary to reach
@@ -874,7 +828,7 @@ static int amdgpu_dri2_get_msc(DrawablePtr draw, CARD64 * ust, CARD64 * msc)
 	xf86CrtcPtr crtc = amdgpu_dri2_drawable_crtc(draw, TRUE);
 
 	/* Drawable not displayed, make up a value */
-	if (crtc == NULL) {
+	if (!crtc) {
 		*ust = 0;
 		*msc = 0;
 		return TRUE;
@@ -919,13 +873,15 @@ CARD32 amdgpu_dri2_deferred_event(OsTimerPtr timer, CARD32 now, pointer data)
 
 	scrn = crtc->scrn;
 	pAMDGPUEnt = AMDGPUEntPriv(scrn);
+	drmmode_crtc = event_info->crtc->driver_private;
 	ret = drmmode_get_current_ust(pAMDGPUEnt->fd, &drm_now);
 	if (ret) {
 		xf86DrvMsg(scrn->scrnIndex, X_ERROR,
 			   "%s cannot get current time\n", __func__);
 		if (event_info->drm_queue_seq)
-			amdgpu_drm_queue_handler(pAMDGPUEnt->fd, 0, 0, 0,
-						 (void*)event_info->drm_queue_seq);
+			drmmode_crtc->drmmode->event_context.
+				vblank_handler(pAMDGPUEnt->fd, 0, 0, 0,
+					       (void*)event_info->drm_queue_seq);
 		else
 			amdgpu_dri2_frame_event_handler(crtc, 0, 0, data);
 		return 0;
@@ -934,15 +890,15 @@ CARD32 amdgpu_dri2_deferred_event(OsTimerPtr timer, CARD32 now, pointer data)
 	 * calculate the frame number from current time
 	 * that would come from CRTC if it were running
 	 */
-	drmmode_crtc = event_info->crtc->driver_private;
 	delta_t = drm_now - (CARD64) drmmode_crtc->dpms_last_ust;
 	delta_seq = delta_t * drmmode_crtc->dpms_last_fps;
 	delta_seq /= 1000000;
 	frame = (CARD64) drmmode_crtc->dpms_last_seq + delta_seq;
 	if (event_info->drm_queue_seq)
-		amdgpu_drm_queue_handler(pAMDGPUEnt->fd, frame, drm_now / 1000000,
-					 drm_now % 1000000,
-					 (void*)event_info->drm_queue_seq);
+		drmmode_crtc->drmmode->event_context.
+			vblank_handler(pAMDGPUEnt->fd, frame, drm_now / 1000000,
+				       drm_now % 1000000,
+				       (void*)event_info->drm_queue_seq);
 	else
 		amdgpu_dri2_frame_event_handler(crtc, frame, drm_now, data);
 	return 0;
@@ -971,13 +927,11 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 {
 	ScreenPtr screen = draw->pScreen;
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
-	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
 	DRI2FrameEventPtr wait_info = NULL;
 	uintptr_t drm_queue_seq = 0;
 	xf86CrtcPtr crtc = amdgpu_dri2_drawable_crtc(draw, TRUE);
 	uint32_t msc_delta;
-	drmVBlank vbl;
-	int ret;
+	uint32_t seq;
 	CARD64 current_msc;
 
 	/* Truncate to match kernel interfaces; means occasional overflow
@@ -987,7 +941,7 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 	remainder &= 0xffffffff;
 
 	/* Drawable not visible, return immediately */
-	if (crtc == NULL)
+	if (!crtc)
 		goto out_complete;
 
 	msc_delta = amdgpu_get_msc_delta(draw, crtc);
@@ -1016,23 +970,19 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 	}
 
 	/* Get current count */
-	vbl.request.type = DRM_VBLANK_RELATIVE;
-	vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
-	vbl.request.sequence = 0;
-	ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-	if (ret) {
+	if (!drmmode_wait_vblank(crtc, DRM_VBLANK_RELATIVE, 0, 0, NULL, &seq)) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "get vblank counter failed: %s\n", strerror(errno));
 		goto out_complete;
 	}
 
-	current_msc = vbl.reply.sequence + msc_delta;
+	current_msc = seq + msc_delta;
 	current_msc &= 0xffffffff;
 
 	drm_queue_seq = amdgpu_drm_queue_alloc(crtc, client, AMDGPU_DRM_QUEUE_ID_DEFAULT,
 					       wait_info, amdgpu_dri2_frame_event_handler,
 					       amdgpu_dri2_frame_event_abort);
-	if (!drm_queue_seq) {
+	if (drm_queue_seq == AMDGPU_DRM_QUEUE_ERROR) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "Allocating DRM queue event entry failed.\n");
 		goto out_complete;
@@ -1053,12 +1003,9 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 		 */
 		if (current_msc >= target_msc)
 			target_msc = current_msc;
-		vbl.request.type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
-		vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
-		vbl.request.sequence = target_msc - msc_delta;
-		vbl.request.signal = drm_queue_seq;
-		ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-		if (ret) {
+		if (!drmmode_wait_vblank(crtc, DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT,
+					 target_msc - msc_delta, drm_queue_seq, NULL,
+					 NULL)) {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 				   "get vblank counter failed: %s\n",
 				   strerror(errno));
@@ -1073,11 +1020,7 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 	 * If we get here, target_msc has already passed or we don't have one,
 	 * so we queue an event that will satisfy the divisor/remainder equation.
 	 */
-	vbl.request.type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
-	vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
-
-	vbl.request.sequence = current_msc - (current_msc % divisor) +
-	    remainder - msc_delta;
+	target_msc = current_msc - (current_msc % divisor) + remainder - msc_delta;
 
 	/*
 	 * If calculated remainder is larger than requested remainder,
@@ -1086,11 +1029,10 @@ static int amdgpu_dri2_schedule_wait_msc(ClientPtr client, DrawablePtr draw,
 	 * that will happen.
 	 */
 	if ((current_msc % divisor) >= remainder)
-		vbl.request.sequence += divisor;
+		target_msc += divisor;
 
-	vbl.request.signal = drm_queue_seq;
-	ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-	if (ret) {
+	if (!drmmode_wait_vblank(crtc, DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT,
+				 target_msc, drm_queue_seq, NULL, NULL)) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "get vblank counter failed: %s\n", strerror(errno));
 		goto out_complete;
@@ -1134,14 +1076,14 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 {
 	ScreenPtr screen = draw->pScreen;
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
-	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
 	xf86CrtcPtr crtc = amdgpu_dri2_drawable_crtc(draw, TRUE);
 	uint32_t msc_delta;
-	drmVBlank vbl;
-	int ret, flip = 0;
+	drmVBlankSeqType type;
+	uint32_t seq;
+	int flip = 0;
 	DRI2FrameEventPtr swap_info = NULL;
 	uintptr_t drm_queue_seq;
-	CARD64 current_msc;
+	CARD64 current_msc, event_msc;
 	BoxRec box;
 	RegionRec region;
 
@@ -1159,7 +1101,7 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	amdgpu_dri2_ref_buffer(back);
 
 	/* either off-screen or CRTC not usable... just complete the swap */
-	if (crtc == NULL)
+	if (!crtc)
 		goto blit_fallback;
 
 	msc_delta = amdgpu_get_msc_delta(draw, crtc);
@@ -1180,7 +1122,7 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	drm_queue_seq = amdgpu_drm_queue_alloc(crtc, client, AMDGPU_DRM_QUEUE_ID_DEFAULT,
 					       swap_info, amdgpu_dri2_frame_event_handler,
 					       amdgpu_dri2_frame_event_abort);
-	if (!drm_queue_seq) {
+	if (drm_queue_seq == AMDGPU_DRM_QUEUE_ERROR) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "Allocating DRM queue entry failed.\n");
 		goto blit_fallback;
@@ -1204,22 +1146,18 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	}
 
 	/* Get current count */
-	vbl.request.type = DRM_VBLANK_RELATIVE;
-	vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
-	vbl.request.sequence = 0;
-	ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-	if (ret) {
+	if (!drmmode_wait_vblank(crtc, DRM_VBLANK_RELATIVE, 0, 0, NULL, &seq)) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "first get vblank counter failed: %s\n",
 			   strerror(errno));
 		goto blit_fallback;
 	}
 
-	current_msc = vbl.reply.sequence + msc_delta;
+	current_msc = seq + msc_delta;
 	current_msc &= 0xffffffff;
 
 	/* Flips need to be submitted one frame before */
-	if (can_flip(scrn, draw, front, back)) {
+	if (can_flip(crtc, draw, front, back)) {
 		swap_info->type = DRI2_FLIP;
 		flip = 1;
 	}
@@ -1237,14 +1175,13 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	 * the swap.
 	 */
 	if (divisor == 0 || current_msc < *target_msc) {
-		vbl.request.type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
+		type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
 		/* If non-pageflipping, but blitting/exchanging, we need to use
 		 * DRM_VBLANK_NEXTONMISS to avoid unreliable timestamping later
 		 * on.
 		 */
 		if (flip == 0)
-			vbl.request.type |= DRM_VBLANK_NEXTONMISS;
-		vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
+			type |= DRM_VBLANK_NEXTONMISS;
 
 		/* If target_msc already reached or passed, set it to
 		 * current_msc to ensure we return a reasonable value back
@@ -1253,17 +1190,15 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 		if (current_msc >= *target_msc)
 			*target_msc = current_msc;
 
-		vbl.request.sequence = *target_msc - msc_delta;
-		vbl.request.signal = drm_queue_seq;
-		ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-		if (ret) {
+		if (!drmmode_wait_vblank(crtc, type, *target_msc - msc_delta,
+					 drm_queue_seq, NULL, &seq)) {
 			xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 				   "divisor 0 get vblank counter failed: %s\n",
 				   strerror(errno));
 			goto blit_fallback;
 		}
 
-		*target_msc = vbl.reply.sequence + flip + msc_delta;
+		*target_msc = seq + flip + msc_delta;
 		*target_msc &= 0xffffffff;
 		swap_info->frame = *target_msc;
 
@@ -1275,13 +1210,11 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	 * and we need to queue an event that will satisfy the divisor/remainder
 	 * equation.
 	 */
-	vbl.request.type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
+	type = DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
 	if (flip == 0)
-		vbl.request.type |= DRM_VBLANK_NEXTONMISS;
-	vbl.request.type |= amdgpu_populate_vbl_request_type(crtc);
+		type |= DRM_VBLANK_NEXTONMISS;
 
-	vbl.request.sequence = current_msc - (current_msc % divisor) +
-	    remainder - msc_delta;
+	event_msc = current_msc - (current_msc % divisor) + remainder - msc_delta;
 
 	/*
 	 * If the calculated deadline vbl.request.sequence is smaller than
@@ -1294,15 +1227,13 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	 * into account, as well as a potential DRM_VBLANK_NEXTONMISS delay
 	 * if we are blitting/exchanging instead of flipping.
 	 */
-	if (vbl.request.sequence <= current_msc)
-		vbl.request.sequence += divisor;
+	if (event_msc <= current_msc)
+		event_msc += divisor;
 
 	/* Account for 1 frame extra pageflip delay if flip > 0 */
-	vbl.request.sequence -= flip;
+	event_msc -= flip;
 
-	vbl.request.signal = drm_queue_seq;
-	ret = drmWaitVBlank(pAMDGPUEnt->fd, &vbl);
-	if (ret) {
+	if (!drmmode_wait_vblank(crtc, type, event_msc, drm_queue_seq, NULL, &seq)) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
 			   "final get vblank counter failed: %s\n",
 			   strerror(errno));
@@ -1310,7 +1241,7 @@ static int amdgpu_dri2_schedule_swap(ClientPtr client, DrawablePtr draw,
 	}
 
 	/* Adjust returned value for 1 fame pageflip offset of flip > 0 */
-	*target_msc = vbl.reply.sequence + flip + msc_delta;
+	*target_msc = seq + flip + msc_delta;
 	*target_msc &= 0xffffffff;
 	swap_info->frame = *target_msc;
 
@@ -1327,7 +1258,7 @@ blit_fallback:
 		box.y2 = draw->height;
 		REGION_INIT(pScreen, &region, &box, 0);
 
-		amdgpu_dri2_copy_region(draw, &region, front, back);
+		amdgpu_dri2_copy_region2(draw->pScreen, draw, &region, front, back);
 
 		DRI2SwapComplete(client, draw, 0, 0, 0, DRI2_BLIT_COMPLETE, func, data);
 
@@ -1356,13 +1287,8 @@ Bool amdgpu_dri2_screen_init(ScreenPtr pScreen)
 	dri2_info.driverName = SI_DRIVER_NAME;
 	dri2_info.fd = pAMDGPUEnt->fd;
 	dri2_info.deviceName = info->dri2.device_name;
-	dri2_info.version = DRI2INFOREC_VERSION;
-	dri2_info.CreateBuffer = amdgpu_dri2_create_buffer;
-	dri2_info.DestroyBuffer = amdgpu_dri2_destroy_buffer;
-	dri2_info.CopyRegion = amdgpu_dri2_copy_region;
 
 	if (info->drmmode.count_crtcs > 2) {
-#ifdef DRM_CAP_VBLANK_HIGH_CRTC
 		uint64_t cap_value;
 
 		if (drmGetCap
@@ -1377,20 +1303,13 @@ Bool amdgpu_dri2_screen_init(ScreenPtr pScreen)
 				   "handle VBLANKs on CRTC > 1\n");
 			scheduling_works = FALSE;
 		}
-#else
-		xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-			   "You need to rebuild against a "
-			   "newer libdrm to handle VBLANKs on CRTC > 1\n");
-		scheduling_works = FALSE;
-#endif
 	}
 
 	if (scheduling_works) {
-		dri2_info.version = 4;
 		dri2_info.ScheduleSwap = amdgpu_dri2_schedule_swap;
 		dri2_info.GetMSC = amdgpu_dri2_get_msc;
 		dri2_info.ScheduleWaitMSC = amdgpu_dri2_schedule_wait_msc;
-		dri2_info.numDrivers = AMDGPU_ARRAY_SIZE(driverNames);
+		dri2_info.numDrivers = ARRAY_SIZE(driverNames);
 		dri2_info.driverNames = driverNames;
 		driverNames[0] = driverNames[1] = dri2_info.driverName;
 
@@ -1410,12 +1329,10 @@ Bool amdgpu_dri2_screen_init(ScreenPtr pScreen)
 		DRI2InfoCnt++;
 	}
 
-#if DRI2INFOREC_VERSION >= 9
 	dri2_info.version = 9;
 	dri2_info.CreateBuffer2 = amdgpu_dri2_create_buffer2;
 	dri2_info.DestroyBuffer2 = amdgpu_dri2_destroy_buffer2;
 	dri2_info.CopyRegion2 = amdgpu_dri2_copy_region2;
-#endif
 
 	info->dri2.enabled = DRI2ScreenInit(pScreen, &dri2_info);
 	return info->dri2.enabled;
