@@ -40,9 +40,7 @@
 #include <errno.h>
 #include <libgen.h>
 
-
-static int
-amdgpu_dri3_open(ScreenPtr screen, RRProviderPtr provider, int *out)
+static int open_master_node(ScreenPtr screen, int *out)
 {
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
 	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
@@ -88,6 +86,36 @@ amdgpu_dri3_open(ScreenPtr screen, RRProviderPtr provider, int *out)
 	return Success;
 }
 
+static int open_render_node(ScreenPtr screen, int *out)
+{
+	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
+	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
+	int fd;
+
+	fd = open(pAMDGPUEnt->render_node, O_RDWR | O_CLOEXEC);
+	if (fd < 0)
+		return BadAlloc;
+
+	*out = fd;
+	return Success;
+}
+
+static int
+amdgpu_dri3_open(ScreenPtr screen, RRProviderPtr provider, int *out)
+{
+	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
+	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
+	int ret = BadAlloc;
+
+	if (pAMDGPUEnt->render_node)
+		ret = open_render_node(screen, out);
+
+	if (ret != Success)
+		ret = open_master_node(screen, out);
+
+	return ret;
+}
+
 #if DRI3_SCREEN_INFO_VERSION >= 1 && XORG_VERSION_CURRENT <= XORG_VERSION_NUMERIC(1,18,99,1,0)
 
 static int
@@ -140,6 +168,7 @@ static PixmapPtr amdgpu_dri3_pixmap_from_fd(ScreenPtr screen,
 
 			if (priv) {
 				amdgpu_set_pixmap_private(pixmap, priv);
+				pixmap->usage_hint |= AMDGPU_CREATE_PIXMAP_DRI2;
 				return pixmap;
 			}
 
@@ -229,6 +258,9 @@ Bool
 amdgpu_dri3_screen_init(ScreenPtr screen)
 {
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
+	AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
+
+	pAMDGPUEnt->render_node = drmGetRenderDeviceNameFromFd(pAMDGPUEnt->fd);
 
 	if (!dri3_screen_init(screen, &amdgpu_dri3_screen_info)) {
 		xf86DrvMsg(scrn->scrnIndex, X_WARNING,
